/*
 * keywork.c 
 *
 * Basic keyboard input routines for User Interface Generator
 * Betsy A. Gamrat,  8/15/94
 * (Compile with screen.c)
 */

#include <bios.h>
#include <ctype.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "pckeys.h"
#include "keywork.h"

#define FALSE 0
#define TRUE 1

int get_key (void)
{
  return (bioskey(0));
}

int get_string (int x, int y, char *str, int len, char type)
{
  int insert_mode = FALSE;
  int i, j, key, local_len, done, good_key, decimal_point;
  char local_str[80], *ptr;

  /* Check for valid length range */
  if ((len < MIN_LEN) || (len > MAX_LEN)) return (NO_KEY);

  /* Check for valid type */
  if ((type != REAL) && (type != INTEGER) && (type != HEX) &&
                  (type != CHAR) && (type != ANY) && (type != ALPHA))
      return (NO_KEY);

  /* Check for valid X */
  if ((x < MIN_X) || (x > MAX_X)) return (NO_KEY);

  /* Check for valid Y */
  if ((y < MIN_Y) || (y > MAX_Y)) return (NO_KEY);

  /* Initialize local variables */
  done = FALSE;
  strcpy (&local_str[0],str);
  decimal_point = (strchr(&local_str[0],'.') != NULL);
  i = local_len = strlen (&local_str[0]);
  if (i == len)
    i--;

  /* Position cursor at beginning of field */
  gotoxy (x,y);

  /* Display text passed in */
  puts (&local_str[0]);

  /* Position cursor at end of text */
  gotoxy (x+i,y);

  /* Process keystrokes */
  while (!done)
  {
    key = get_key();

    /* if this is not a function key */
    if (key & 0xff)
    {
      switch (key)
      {
        case ENTER:
ENTER_CODE:
          ptr = &local_str[0];
          ptr = &local_str[strlen(&local_str[0])];

          /* Remove trailing spaces */
          ptr--;
          while (*ptr == ' ') ptr--;
          ptr++;
          *ptr = '\0';

          strncpy (str,&local_str[0],len);

          done = TRUE;
          break;
        case BS:
          if (i < (local_len-1))
          {
            /* Mid-entry backspace is actually a delete */
            i--;
            gotoxy (x+i,y);
            key = DELETE;
          }
          /* Fall through to delete code */
          goto DELETE_CODE;
        case ESC:
          done = TRUE;
          break;
        case TAB:
          goto ENTER_CODE;
        default:
        {
          /* Evaluate alphanumeric key */
          good_key = FALSE;
          key &= 0xff;
          switch (type)
          {
            case CHAR:
              good_key = isalpha (key) ? TRUE : FALSE;
              break;
            case REAL:
              /* A real number has digits and one decimal point */
              if (((decimal_point == FALSE) || (local_str[i] == '.')) &&
                  (key == '.'))
              {
                good_key = TRUE;
                decimal_point = TRUE;
              }
              if (good_key || (key == '.'))
                break;
              /* Fall through */
            case INTEGER:
              good_key = isdigit(key) ? TRUE : FALSE;
              break;
            case HEX:
              good_key = isxdigit (key) ? TRUE : FALSE;
              break;
            case ANY:
              good_key = TRUE;
              break;
            case ALPHA:
              good_key = isalnum (key) ? TRUE : FALSE;
              break;
          }

          /* If the key was good */
          if (good_key == TRUE)
          {
            /* If insert_mode is on, and this is not an empty entry */
            if ((insert_mode == TRUE) && (local_len > 0))
            {
              /* Insert the new key into the local string */
              for (j = local_len; j > i; j--)
                local_str [j] = local_str [j-1];
              local_str [i] = key;
              if (i < (len-1)) i++;
              if (local_len < len) local_len++;
              local_str [local_len] = '\0';

              /* Display the new string */
              gotoxy (x,y);
              puts (&local_str[0]);
            }
            else        /* Overwrite */
            {
              /* Add key to local string */
              local_str[i] = key;

              /* Update display */
              putchar (key);

              /* Increment local length and current cursor position */
              if (local_len < len) local_len++;
              if (i < (len-1)) i++;
            }
            /* Position cursor at current entry point */
            gotoxy (x+i,y);

            /* Ensure end of string marker is present */
            local_str [local_len] = '\0';
          }
        }
      }
    }
    else        /* Function key */
    {
      switch (key)
      {
        case INSERT:
          /* Toggle */
          insert_mode = (insert_mode == TRUE) ? FALSE : TRUE;
          break;
        case DELETE:
DELETE_CODE:
          if (local_len > 0)
          {
            /* In case this is a REAL entry, toggle the decimal point flag */
            if (local_str[i-1] == '.')
              decimal_point = FALSE;

            /* Remove the character */
            for (j = i; j < local_len; j++)
              local_str [j] = local_str [j+1];
            local_len--;
            local_str [local_len] = local_str [local_len+1];

            /* Reposition the cursor */
            if (((key == BS) && (i > 0)) ||
                ((key == DELETE) && (i > local_len)))
              i--;

            /* Display the new string */
            gotoxy (x,y);
            printf ("%s",&local_str[0]);
            putchar (' ');
            gotoxy (x+i,y);
          }
          break;
        case RIGHT_ARROW:
          if (local_len > 0)
          {
            if (i < (len-1))
            {
              i++;
              gotoxy (x+i,y);
            }
            else
              /* Assume that the user wants to leave this field */
              done = TRUE;      
          }
          break;
        case LEFT_ARROW:
          if (i > 0)
          {
            i--;
            gotoxy (x+i,y);
          }
          else
            /* Assume that the user wants to leave this field */
            done = TRUE;
          break;
        case HOME:
          if (local_len > 0)
          {
            i = 0;
            gotoxy (x,y);
          }
          break;
        case END:
          if (local_len > 0)
          {
            i = local_len-1;
            gotoxy (x+i,y);
          }
          break;
        case SHIFT_TAB:
        case UP_ARROW:
        case DOWN_ARROW:
        case F9:
        case F10:
            /* Assume that the user wants to leave this field */
           goto ENTER_CODE;
        case F1:
        case F2:
        case F3:
        case F4:
        case F5:
        case F6:
        case F7:
        case F8:
        case PAGE_UP:
        case PAGE_DOWN:
          done = TRUE;
          break;
      }
    }
  }

  return key;
}
